"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.emit = void 0;
const Keywords = require("../tokens/keywords");
const Delimiters = require("../tokens/delimiters");
const GenericTokens = require("../tokens/generic_tokens");
const ScopeTokens = require("../tokens/scope_tokens");
const TypeReferenceTokens = require("../tokens/type_reference_tokens");
const AttributeEmitter = require("./attribute_emitter");
const FieldEmitter = require("./field_emitter");
const PropertyEmitter = require("./property_emitter");
const MethodEmitter = require("./method_emitter");
function emit(writer, _class) {
    const noFields = _class.fields == null || _class.fields.length === 0;
    const noProperties = _class.properties == null || _class.properties.length === 0;
    AttributeEmitter.emitMany(writer, _class.attributes);
    emitDefinition(writer, _class);
    FieldEmitter.emitMany(writer, _class.fields, true);
    PropertyEmitter.emitMany(writer, _class.properties, noFields);
    MethodEmitter.emitMany(writer, _class.methods, noFields && noProperties);
    emitEnd(writer);
}
exports.emit = emit;
function emitDefinition(writer, _class) {
    const { access, isNew, isPartial, modifier, name, generics, baseType } = _class;
    const accessTokens = access ? [access, Delimiters.space] : [];
    const newTokens = isNew ? [Keywords._new, Delimiters.space] : [];
    const modifierTokens = modifier ? [modifier, Delimiters.space] : [];
    const partialTokens = isPartial ? [Keywords.partial, Delimiters.space] : [];
    const genericTokens = GenericTokens.emit(generics);
    const baseTypeTokens = baseType ? TypeReferenceTokens.emit(baseType) : [];
    const extensionTokens = baseTypeTokens.length ? [Delimiters.space, Delimiters.colon, Delimiters.space, ...baseTypeTokens] : [];
    writer
        .outputTabs()
        .write(...accessTokens)
        .write(...newTokens)
        .write(...modifierTokens)
        .write(...partialTokens)
        .write(Keywords._class, Delimiters.space, name)
        .write(...genericTokens)
        .write(...extensionTokens)
        .writeLineNoTabs()
        .writeLine(ScopeTokens.open)
        .indent();
}
function emitEnd(writer) {
    writer
        .unindent()
        .writeLine(ScopeTokens.close);
}
