"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.emit = void 0;
const Keywords = require("../tokens/keywords");
const Delimiters = require("../tokens/delimiters");
const GenericTokens = require("../tokens/generic_tokens");
const ScopeTokens = require("../tokens/scope_tokens");
const AttributeEmitter = require("./attribute_emitter");
const FieldEmitter = require("./field_emitter");
const PropertyEmitter = require("./property_emitter");
const MethodEmitter = require("./method_emitter");
function emit(writer, struct) {
    const noFields = struct.fields == null || struct.fields.length === 0;
    const noProperties = struct.properties == null || struct.properties.length === 0;
    AttributeEmitter.emitMany(writer, struct.attributes);
    emitDefinition(writer, struct);
    FieldEmitter.emitMany(writer, struct.fields, true);
    PropertyEmitter.emitMany(writer, struct.properties, noFields);
    MethodEmitter.emitMany(writer, struct.methods, noFields && noProperties);
    emitEnd(writer);
}
exports.emit = emit;
function emitDefinition(writer, struct) {
    const { access, isNew, isPartial, name, generics } = struct;
    const accessTokens = access ? [access, Delimiters.space] : [];
    const newTokens = isNew ? [Keywords._new, Delimiters.space] : [];
    const partialTokens = isPartial ? [Keywords.partial, Delimiters.space] : [];
    const genericTokens = GenericTokens.emit(generics);
    writer
        .outputTabs()
        .write(...accessTokens)
        .write(...newTokens)
        .write(...partialTokens)
        .write(Keywords.struct, Delimiters.space, name)
        .write(...genericTokens)
        .writeLineNoTabs()
        .writeLine(ScopeTokens.open)
        .indent();
}
function emitEnd(writer) {
    writer
        .unindent()
        .writeLine(ScopeTokens.close);
}
